<?php
/************************************************************************
 * Mailer v0.2.1-FINAL                                Start: 04/04/2009 *
 * ===================                          Last change: 04/04/2009 *
 *                                                                      *
 * -------------------------------------------------------------------- *
 * File              : wrapper-functions.php                            *
 * -------------------------------------------------------------------- *
 * Short description : Wrapper functions                                *
 * -------------------------------------------------------------------- *
 * Kurzbeschreibung  : Wrapper-Funktionen                               *
 * -------------------------------------------------------------------- *
 * $Revision::                                                        $ *
 * $Date::                                                            $ *
 * $Tag:: 0.2.1-FINAL                                                 $ *
 * $Author::                                                          $ *
 * Needs to be in all Files and every File needs "svn propset           *
 * svn:keywords Date Revision" (autoprobset!) at least!!!!!!            *
 * -------------------------------------------------------------------- *
 * Copyright (c) 2003 - 2009 by Roland Haeder                           *
 * Copyright (c) 2009, 2010 by Mailer Developer Team                    *
 * For more information visit: http://www.mxchange.org                  *
 *                                                                      *
 * This program is free software; you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation; either version 2 of the License, or    *
 * (at your option) any later version.                                  *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program; if not, write to the Free Software          *
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,               *
 * MA  02110-1301  USA                                                  *
 ************************************************************************/

// Some security stuff...
if (!defined('__SECURITY')) {
	die();
} // END - if

// Read a given file
function readFromFile ($FQFN) {
	// Sanity-check if file is there (should be there, but just to make it sure)
	if (!isFileReadable($FQFN)) {
		// This should not happen
		debug_report_bug(__FUNCTION__.': File ' . basename($FQFN) . ' is not readable!');
	} // END - if

	// Is it cached?
	if (!isset($GLOBALS['file_content'][$FQFN])) {
		// Load the file
		if (function_exists('file_get_contents')) {
			// Use new function
			$GLOBALS['file_content'][$FQFN] = file_get_contents($FQFN);
		} else {
			// Fall-back to implode-file chain
			$GLOBALS['file_content'][$FQFN] = implode('', file($FQFN));
		}
	} // END - if

	// Return the content
	return $GLOBALS['file_content'][$FQFN];
}

// Writes content to a file
function writeToFile ($FQFN, $content, $aquireLock = false) {
	// Is the file writeable?
	if ((isFileReadable($FQFN)) && (!is_writeable($FQFN)) && (!changeMode($FQFN, 0644))) {
		// Not writeable!
		logDebugMessage(__FUNCTION__, __LINE__, sprintf("File %s not writeable.", basename($FQFN)));

		// Failed! :(
		return false;
	} // END - if

	// By default all is failed...
	$return = false;

	// Is the function there?
	if (function_exists('file_put_contents')) {
		// With lock?
		if ($aquireLock === true) {
			// Write it directly with lock
			$return = file_put_contents($FQFN, $content, LOCK_EX);
		} else {
			// Write it directly
			$return = file_put_contents($FQFN, $content);
		}
	} else {
		// Write it with fopen
		$fp = fopen($FQFN, 'w') or debug_report_bug(__FUNCTION__, __LINE__, 'Cannot write to file ' . basename($FQFN) . '!');

		// Aquire lock
		if ($aquireLock === true) flock($fp, LOCK_EX);

		// Write content
		fwrite($fp, $content);

		// Close stream
		fclose($fp);
	}

	// Mark it as readable
	$GLOBALS['file_readable'][$FQFN] = true;

	// Remember content in cache
	$GLOBALS['file_content'][$FQFN] = $content;

	// Return status
	return changeMode($FQFN, 0644);
}

// Clears the output buffer. This function does *NOT* backup sent content.
function clearOutputBuffer () {
	// Trigger an error on failure
	if (!ob_end_clean()) {
		// Failed!
		debug_report_bug(__FUNCTION__.': Failed to clean output buffer.');
	} // END - if
}

// Encode strings
// @TODO Implement $compress
function encodeString ($str, $compress = true) {
	$str = urlencode(base64_encode(compileUriCode($str)));
	return $str;
}

// Decode strings encoded with encodeString()
// @TODO Implement $decompress
function decodeString ($str, $decompress = true) {
	$str = compileUriCode(base64_decode(urldecode(compileUriCode($str))));
	return $str;
}

// Decode entities in a nicer way
function decodeEntities ($str, $quote = ENT_NOQUOTES) {
	// Decode the entities to UTF-8 now
	$decodedString = html_entity_decode($str, $quote, 'UTF-8');

	// Return decoded string
	return $decodedString;
}

// Merges an array together but only if both are arrays
function merge_array ($array1, $array2) {
	// Are both an array?
	if ((!is_array($array1)) && (!is_array($array2))) {
		// Both are not arrays
		debug_report_bug(__FUNCTION__ . ': No arrays provided!');
	} elseif (!is_array($array1)) {
		// Left one is not an array
		debug_report_bug(__FILE__, __LINE__, sprintf("array1 is not an array. array != %s", gettype($array1)));
	} elseif (!is_array($array2)) {
		// Right one is not an array
		debug_report_bug(__FILE__, __LINE__, sprintf("array2 is not an array. array != %s", gettype($array2)));
	}

	// Merge all together
	return array_merge($array1, $array2);
}

// Check if given FQFN is a readable file
function isFileReadable ($FQFN) {
	// Do we have cache?
	if (!isset($GLOBALS['file_readable'][$FQFN])) {
		// Check all...
		$GLOBALS['file_readable'][$FQFN] = ((file_exists($FQFN)) && (is_file($FQFN)) && (is_readable($FQFN)));

		// Debug message
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'file=' . basename($FQFN) . ' - CHECK! (' . intval($GLOBALS['file_readable'][$FQFN]) . ')');
	} else {
		// Cache used
		//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'file=' . basename($FQFN) . ' - CACHE! (' . intval($GLOBALS['file_readable'][$FQFN]) . ')');
	}

	// Return result
	return $GLOBALS['file_readable'][$FQFN];
}

// Checks wether the given FQFN is a directory and not ., .. or .svn
function isDirectory ($FQFN) {
	// Do we have cache?
	if (!isset($GLOBALS['is_directory'][$FQFN])) {
		// Generate baseName
		$baseName = basename($FQFN);

		// Check it
		$GLOBALS['is_directory'][$FQFN] = ((is_dir($FQFN)) && ($baseName != '.') && ($baseName != '..') && ($baseName != '.svn'));
	} // END - if

	// Return the result
	return $GLOBALS['is_directory'][$FQFN];
}

// "Getter" for remote IP number
function detectRemoteAddr () {
	// Get remote ip from environment
	$remoteAddr = determineRealRemoteAddress();

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteAddr = getAnonymousRemoteAddress($remoteAddr);
	} // END - if

	// Return it
	return $remoteAddr;
}

// "Getter" for remote hostname
function detectRemoteHostname () {
	// Get remote ip from environment
	$remoteHost = getenv('REMOTE_HOST');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$remoteHost = getAnonymousRemoteHost($remoteHost);
	} // END - if

	// Return it
	return $remoteHost;
}

// "Getter" for user agent
function detectUserAgent ($alwaysReal = false) {
	// Get remote ip from environment
	$userAgent = getenv('HTTP_USER_AGENT');

	// Is removeip installed?
	if ((isExtensionActive('removeip')) && ($alwaysReal === false)) {
		// Then anonymize it
		$userAgent = getAnonymousUserAgent($userAgent);
	} // END - if

	// Return it
	return $userAgent;
}

// "Getter" for referer
function detectReferer () {
	// Get remote ip from environment
	$referer = getenv('HTTP_REFERER');

	// Is removeip installed?
	if (isExtensionActive('removeip')) {
		// Then anonymize it
		$referer = getAnonymousReferer($referer);
	} // END - if

	// Return it
	return $referer;
}

// "Getter" for request URI
function detectRequestUri () {
	// Return it
	return (getenv('REQUEST_URI'));
}

// "Getter" for query string
function detectQueryString () {
	return str_replace('&', '&amp;', (getenv('QUERY_STRING')));
}

// "Getter" for SERVER_NAME
function detectServerName () {
	// Return it
	return (getenv('SERVER_NAME'));
}

// Check wether we are installing
function isInstalling () {
	// Determine wether we are installing
	if (!isset($GLOBALS['mailer_installing'])) {
		// Check URL (css.php/js.php need this)
		$GLOBALS['mailer_installing'] = isGetRequestParameterSet('installing');
	} // END - if

	// Return result
	return $GLOBALS['mailer_installing'];
}

// Check wether this script is installed
function isInstalled () {
	// Do we have cache?
	if (!isset($GLOBALS['is_installed'])) {
		// Determine wether this script is installed
		$GLOBALS['is_installed'] = (
		(
			// First is config
			(
				(
					isConfigEntrySet('MXCHANGE_INSTALLED')
				) && (
					getConfig('MXCHANGE_INSTALLED') == 'Y'
				)
			)
		) || (
			// New config file found and loaded
			isIncludeReadable(getConfig('CACHE_PATH') . 'config-local.php')
		) || (
			(
				// New config file found, but not yet read
				isIncludeReadable(getConfig('CACHE_PATH') . 'config-local.php')
			) && (
				(
					// Only new config file is found
					!isIncludeReadable('inc/config.php')
				) || (
					// Is installation mode
					!isInstalling()
				)
			)
		));
	} // END - if

	// Then use the cache
	return $GLOBALS['is_installed'];
}

// Check wether an admin is registered
function isAdminRegistered () {
	// Is cache set?
	if (!isset($GLOBALS['is_admin_registered'])) {
		// Simply check it
		$GLOBALS['is_admin_registered'] = ((isConfigEntrySet('ADMIN_REGISTERED')) && (getConfig('ADMIN_REGISTERED') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS['is_admin_registered'];
}

// Checks wether the reset mode is active
function isResetModeEnabled () {
	// Now simply check it
	return ((isset($GLOBALS['reset_enabled'])) && ($GLOBALS['reset_enabled'] === true));
}

// Checks wether the debug mode is enabled
function isDebugModeEnabled () {
	// Is cache set?
	if (!isset($GLOBALS['is_debugmode_enabled'])) {
		// Simply check it
		$GLOBALS['is_debugmode_enabled'] = ((isConfigEntrySet('DEBUG_MODE')) && (getConfig('DEBUG_MODE') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS['is_debugmode_enabled'];
}

// Checks wether SQL debugging is enabled
function isSqlDebuggingEnabled () {
	// Is cache set?
	if (!isset($GLOBALS['is_sql_debug_enabled'])) {
		// Determine if SQL debugging is enabled
		$GLOBALS['is_sql_debug_enabled'] = ((isConfigEntrySet('DEBUG_SQL')) && (getConfig('DEBUG_SQL') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS['is_sql_debug_enabled'];
}

// Checks wether we shall debug regular expressions
function isDebugRegularExpressionEnabled () {
	// Is cache set?
	if (!isset($GLOBALS['is_regular_exp_debug_enabled'])) {
		// Simply check it
		$GLOBALS['is_regular_exp_debug_enabled'] = ((isConfigEntrySet('DEBUG_REGEX')) && (getConfig('DEBUG_REGEX') == 'Y'));
	} // END - if

	// Return it
	return $GLOBALS['is_regular_exp_debug_enabled'];
}

// Checks wether the cache instance is valid
function isCacheInstanceValid () {
	return ((isset($GLOBALS['cache_instance'])) && (is_object($GLOBALS['cache_instance'])));
}

// Copies a file from source to destination and verifies if that goes fine.
// This function should wrap the copy() command and make a nicer debug backtrace
// even if there is no xdebug extension installed.
function copyFileVerified ($source, $dest, $chmod = '') {
	// Failed is the default
	$status = false;

	// Is the source file there?
	if (!isFileReadable($source)) {
		// Then abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot read from source file ' . basename($source) . '.');
	} // END - if

	// Is the target directory there?
	if (!isDirectory(dirname($dest))) {
		// Then abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot find directory ' . str_replace(getConfig('PATH'), '', dirname($dest)) . '.');
	} // END - if

	// Now try to copy it
	if (!copy($source, $dest)) {
		// Something went wrong
		debug_report_bug(__FUNCTION__, __LINE__, 'copy() has failed to copy the file.');
	} else {
		// Reset cache
		$GLOBALS['file_readable'][$dest] = true;
	}

	// If there are chmod rights set, apply them
	if (!empty($chmod)) {
		// Try to apply them
		$status = changeMode($dest, $chmod);
	} else {
		// All fine
		$status = true;
	}

	// All fine
	return $status;
}

// Wrapper function for header()
// Send a header but checks before if we can do so
function sendHeader ($header) {
	// Send the header
	//* DEBUG: */ logDebugMessage(__FUNCTION__ . ': header=' . $header);
	$GLOBALS['header'][] = trim($header);
}

// Flushes all headers
function flushHeaders () {
	// Is the header already sent?
	if (headers_sent()) {
		// Then abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Headers already sent!');
	} // END - if

	// Flush all headers if found
	if ((isset($GLOBALS['header'])) && (is_array($GLOBALS['header']))) {
		foreach ($GLOBALS['header'] as $header) {
			header($header);
		} // END - foreach
	} // END - if

	// Mark them as flushed
	$GLOBALS['header'] = array();
}

// Wrapper function for chmod()
// @TODO Do some more sanity check here
function changeMode ($FQFN, $mode) {
	// Is the file/directory there?
	if ((!isFileReadable($FQFN)) && (!isDirectory($FQFN))) {
		// Neither, so abort here
		debug_report_bug(__FUNCTION__, __LINE__, 'Cannot chmod() on ' . basename($FQFN) . '.');
	} // END - if

	// Try to set them
	chmod($FQFN, $mode);
}

// Wrapper for unlink()
function removeFile ($FQFN) {
	// Is the file there?
	if (isFileReadable($FQFN)) {
		// Reset cache first
		$GLOBALS['file_readable'][$FQFN] = false;

		// Yes, so remove it
		return unlink($FQFN);
	} // END - if

	// All fine if no file was removed. If we change this to 'false' or rewrite
	// above if() block it would be to restrictive.
	return true;
}

// Wrapper for $_POST['sel']
function countPostSelection ($element = 'sel') {
	// Is it set?
	if (isPostRequestParameterSet($element)) {
		// Return counted elements
		return countSelection(postRequestParameter($element));
	} else {
		// Return zero if not found
		return 0;
	}
}

// Checks wether the config-local.php is loaded
function isConfigLocalLoaded () {
	return ((isset($GLOBALS['config_local_loaded'])) && ($GLOBALS['config_local_loaded'] === true));
}

// Checks wether a nickname or userid was entered and caches the result
function isNicknameUsed ($userid) {
	// Default is false
	$isUsed = false;

	// Is the cache there
	if (isset($GLOBALS['is_nickname_used'][$userid])) {
		// Then use it
		$isUsed = $GLOBALS['is_nickname_used'][$userid];
	} else {
		// Determine it
		$isUsed = (('' . round($userid) . '') != $userid);

		// And write it to the cache
		$GLOBALS['is_nickname_used'][$userid] = $isUsed;
	}

	// Return the result
	return $isUsed;
}

// Getter for 'what' value
function getWhat () {
	// Default is null
	$what = null;

	// Is the value set?
	if (isWhatSet(true)) {
		// Then use it
		$what = $GLOBALS['what'];
	} // END - if

	// Return it
	return $what;
}

// Setter for 'what' value
function setWhat ($newWhat) {
	$GLOBALS['what'] = SQL_ESCAPE($newWhat);
}

// Setter for 'what' from configuration
function setWhatFromConfig ($configEntry) {
	// Get 'what' from config
	$what = getConfig($configEntry);

	// Set it
	setWhat($what);
}

// Checks wether what is set and optionally aborts on miss
function isWhatSet ($strict =  false) {
	// Check for it
	$isset = isset($GLOBALS['what']);

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'what is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'action' value
function getAction ($strict = true) {
	// Default is null
	$action = null;

	// Is the value set?
	if (isActionSet(($strict) && (getOutputMode() == 0))) {
		// Then use it
		$action = $GLOBALS['action'];
	} // END - if

	// Return it
	return $action;
}

// Setter for 'action' value
function setAction ($newAction) {
	$GLOBALS['action'] = SQL_ESCAPE($newAction);
}

// Checks wether action is set and optionally aborts on miss
function isActionSet ($strict =  false) {
	// Check for it
	$isset = ((isset($GLOBALS['action'])) && (!empty($GLOBALS['action'])));

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'action is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Getter for 'module' value
function getModule ($strict = true) {
	// Default is null
	$module = null;

	// Is the value set?
	if (isModuleSet($strict)) {
		// Then use it
		$module = $GLOBALS['module'];
	} // END - if

	// Return it
	return $module;
}

// Setter for 'module' value
function setModule ($newModule) {
	// Secure it and make all modules lower-case
	$GLOBALS['module'] = SQL_ESCAPE(strtolower($newModule));
}

// Checks wether module is set and optionally aborts on miss
function isModuleSet ($strict =  false) {
	// Check for it
	$isset = (!empty($GLOBALS['module']));

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'module is empty.');
	} // END - if

	// Return it
	return (($isset === true) && ($GLOBALS['module'] != 'unknown')) ;
}

// Getter for 'output_mode' value
function getOutputMode () {
	// Default is null
	$output_mode = null;

	// Is the value set?
	if (isOutputModeSet(true)) {
		// Then use it
		$output_mode = $GLOBALS['output_mode'];
	} // END - if

	// Return it
	return $output_mode;
}

// Setter for 'output_mode' value
function setOutputMode ($newOutputMode) {
	$GLOBALS['output_mode'] = (int) $newOutputMode;
}

// Checks wether output_mode is set and optionally aborts on miss
function isOutputModeSet ($strict =  false) {
	// Check for it
	$isset = (isset($GLOBALS['output_mode']));

	// Should we abort here?
	if (($strict === true) && ($isset === false)) {
		// Output backtrace
		debug_report_bug(__FUNCTION__, __LINE__, 'output_mode is empty.');
	} // END - if

	// Return it
	return $isset;
}

// Enables block-mode
function enableBlockMode ($enabled = true) {
	$GLOBALS['block_mode'] = $enabled;
}

// Checks wether block-mode is enabled
function isBlockModeEnabled () {
	// Abort if not set
	if (!isset($GLOBALS['block_mode'])) {
		// Needs to be fixed
		debug_report_bug(__FUNCTION__ . ': block_mode is not set.');
	} // END - if

	// Return it
	return $GLOBALS['block_mode'];
}

// Wrapper function for addPointsThroughReferalSystem()
function addPointsDirectly ($subject, $userid, $points) {
	// Reset level here
	unset($GLOBALS['ref_level']);

	// Call more complicated method (due to more parameters)
	return addPointsThroughReferalSystem($subject, $userid, $points, false, 0, false, 'direct');
}

// Wrapper function to redirect from member-only modules to index
function redirectToIndexMemberOnlyModule () {
	// Do the redirect here
	redirectToUrl('modules.php?module=index&code=' . getCode('MODULE_MEMBER_ONLY') . '&mod=' . getModule());
}

// Wrapper function to redirect to current URL
function redirectToRequestUri () {
	redirectToUrl(basename(detectRequestUri()));
}

// Wrapper function to redirect to de-refered URL
function redirectToDereferedUrl ($URL) {
	// Redirect to to
	redirectToUrl(generateDerefererUrl($URL));
}

// Wrapper function for checking if extension is installed and newer or same version
function isExtensionInstalledAndNewer ($ext_name, $version) {
	// Is an cache entry found?
	if (!isset($GLOBALS['ext_installed_newer'][$ext_name][$version])) {
		$GLOBALS['ext_installed_newer'][$ext_name][$version] = ((isExtensionInstalled($ext_name)) && (getExtensionVersion($ext_name) >= $version));
	} else {
		// Cache hits should be incremented twice
		incrementStatsEntry('cache_hits', 2);
	}

	// Return it
	//* DEBUG: */ print __FUNCTION__.':'.$ext_name.'=&gt;'.$version.':'.intval($GLOBALS['ext_installed_newer'][$ext_name][$version]).'<br />';
	return $GLOBALS['ext_installed_newer'][$ext_name][$version];
}

// Wrapper function for checking if extension is installed and older than given version
function isExtensionInstalledAndOlder ($ext_name, $version) {
	// Is an cache entry found?
	if (!isset($GLOBALS['ext_installed_older'][$ext_name][$version])) {
		$GLOBALS['ext_installed_older'][$ext_name][$version] = ((isExtensionInstalled($ext_name)) && (isExtensionOlder($ext_name, $version)));
	} else {
		// Cache hits should be incremented twice
		incrementStatsEntry('cache_hits', 2);
	}

	// Return it
	//* DEBUG: */ print __FUNCTION__.':'.$ext_name.'&lt;'.$version.':'.intval($GLOBALS['ext_installed_older'][$ext_name][$version]).'<br />';
	return $GLOBALS['ext_installed_older'][$ext_name][$version];
}

// Set username
function setUsername ($userName) {
	$GLOBALS['username'] = (string) $userName;
}

// Get username
function getUsername () {
	// User name set?
	if (!isset($GLOBALS['username'])) {
		// No, so it has to be a guest
		$GLOBALS['username'] = getMessage('USERNAME_GUEST');
	} // END - if

	// Return it
	return $GLOBALS['username'];
}

// Wrapper function for installation phase
function isInstallationPhase () {
	// Do we have cache?
	if (!isset($GLOBALS['installation_phase'])) {
		// Determine it
		$GLOBALS['installation_phase'] = ((!isInstalled()) || (isInstalling()));
	} // END - if

	// Return result
	return $GLOBALS['installation_phase'];
}

// Checks wether the extension demo is actuve and the admin login is demo (password needs to be demo, too!)
function isDemoModeActive () {
	// Is cache set?
	if (!isset($GLOBALS['demo_mode_active'])) {
		// Simply check it
		$GLOBALS['demo_mode_active'] = ((isExtensionActive('demo')) && (getSession('admin_login') == 'demo'));
	} // END - if

	// Return it
	return $GLOBALS['demo_mode_active'];
}

// Getter for PHP caching value
function getPhpCaching () {
	return $GLOBALS['php_caching'];
}

// Checks wether the admin hash is set
function isAdminHashSet ($admin) {
	/**
	 * @TODO Do we really need this check? If yes, try to fix this:
	 * 1.:functions.php:2504, debug_get_mailable_backtrace(0)
	 * 2.:wrapper-functions.php:744, debug_report_bug(1)
	 * 3.:mysql-manager.php:728, isAdminHashSet(1)
	 * 4.:filters.php:384, isAdmin(0)
	 * 5.:debug_get_mailable_backtrace:2457, FILTER_DETERMINE_USERNAME(1)
	 * 6.:filter-functions.php:280, call_user_func_array(2)
	 * 7.:load_cache.php:74, runFilterChain(1)
	 * 8.:inc-functions.php:131, include(1)
	 * 9.:inc-functions.php:145, loadInclude(1)
	 * 10.:mysql-connect.php:104, loadIncludeOnce(1)
	 * 11.:inc-functions.php:131, include(1)
	 * 12.:inc-functions.php:145, loadInclude(1)
	 * 13.:config-global.php:106, loadIncludeOnce(1)
	 * 14.:js.php:57, require(1)
	 */
	if (!isset($GLOBALS['cache_array']['admin'])) {
		debug_report_bug(__FUNCTION__, __LINE__, 'Cache not set.');
	} // END - if

	// Check for admin hash
	return isset($GLOBALS['cache_array']['admin']['password'][$admin]);
}

// Setter for admin hash
function setAdminHash ($admin, $hash) {
	$GLOBALS['cache_array']['admin']['password'][$admin] = $hash;
}

// Init user data array
function initUserData () {
	// User id should not be zero
	if (getCurrentUserId() < 1) debug_report_bug(__FUNCTION__.': User id is zero.');

	// Init the user
	$GLOBALS['user_data'][getCurrentUserId()] = array();
}

// Getter for user data
function getUserData ($column) {
	// User id should not be zero
	if (getCurrentUserId() < 1) debug_report_bug(__FUNCTION__.': User id is zero.');

	// Return the value
	return $GLOBALS['user_data'][getCurrentUserId()][$column];
}

// Geter for whole user data array
function getUserDataArray () {
	// Get user id
	$uid = getCurrentUserId();

	// User id should not be zero
	if ($uid < 1) debug_report_bug(__FUNCTION__.': User id is zero.');

	// Get the whole array if found
	if (isset($GLOBALS['user_data'][$uid])) {
		// Found, so return it
		return $GLOBALS['user_data'][$uid];
	} else {
		// Return empty array
		return array();
	}
}

// Checks if the user data is valid, this may indicate that the user has logged
// in, but you should use isMember() if you want to find that out.
function isUserDataValid () {
	// User id should not be zero so abort here
	if (!isCurrentUserIdSet()) return false;

	// Is it cached?
	if (!isset($GLOBALS['is_userdata_valid'][getCurrentUserId()])) {
		// Determine it
		$GLOBALS['is_userdata_valid'][getCurrentUserId()] = ((isset($GLOBALS['user_data'][getCurrentUserId()])) && (count($GLOBALS['user_data'][getCurrentUserId()]) > 1));
	} // END - if

	// Return the result
	return $GLOBALS['is_userdata_valid'][getCurrentUserId()];
}

// Setter for current userid
function setCurrentUserId ($userid) {
	// Set userid
	$GLOBALS['current_userid'] = bigintval($userid);

	// Unset it to re-determine the actual state
	unset($GLOBALS['is_userdata_valid'][$userid]);
}

// Getter for current userid
function getCurrentUserId () {
	// Userid must be set before it can be used
	if (!isCurrentUserIdSet()) {
		// Not set
		debug_report_bug(__FUNCTION__, __LINE__, 'User id is not set.');
	} // END - if

	// Return the userid
	return $GLOBALS['current_userid'];
}

// Checks if current userid is set
function isCurrentUserIdSet () {
	return ((isset($GLOBALS['current_userid'])) && ($GLOBALS['current_userid'] > 0));
}

// Checks wether we are debugging template cache
function isDebuggingTemplateCache () {
	return (getConfig('DEBUG_TEMPLATE_CACHE') == 'Y');
}

// Wrapper for fetchUserData() and getUserData() calls
function getFetchedUserData ($keyColumn, $userid, $valueColumn) {
	// Is it cached?
	if (!isset($GLOBALS['user_data_cache'][$userid][$keyColumn][$valueColumn])) {
		// Default is 'guest'
		$data = getMessage('USERNAME_GUEST');

		// Can we fetch the user data?
		if (($userid > 0) && (fetchUserData($userid, $keyColumn))) {
			// Now get the data back
			$data = getUserData($valueColumn);
		} // END - if

		// Cache it
		$GLOBALS['user_data_cache'][$userid][$keyColumn][$valueColumn] = $data;
	} // END - if

	// Return it
	return $GLOBALS['user_data_cache'][$userid][$keyColumn][$valueColumn];
}

// Wrapper for strpos() to ease porting from deprecated ereg() function
function isInString ($needle, $haystack) {
	//* DEBUG: */ logDebugMessage(__FUNCTION__, __LINE__, 'needle=' . $needle . ', haystack=' . $haystack . ', returned=' . intval(strpos($haystack, $needle) !== false));
	return (strpos($haystack, $needle) !== false);
}

// Wrapper for strpos() to ease porting from deprecated eregi() function
// This function is case-insensitive
function isInStringIgnoreCase ($needle, $haystack) {
	return (isInString(strtolower($needle), strtolower($haystack)));
}

// Wrapper to check for if fatal errors where detected
function ifFatalErrorsDetected () {
	// Just call the inner function
	return (getTotalFatalErrors() > 0);
}

// Wrapper until we merged to the EL branch
function preCompileCode ($code, $template = '', $compiled = false, $full = true, $overwrite = false) {
	return compileCode($code, false, true, $full);
}

// Setter for HTTP status
function setHttpStatus ($status) {
	$GLOBALS['http_status'] = (string) $status;
}

// Getter for HTTP status
function getHttpStatus () {
	return $GLOBALS['http_status'];
}

// Setter for 'is_template_html'
function enableTemplateHtml ($enable = true) {
	$GLOBALS['is_template_html'] = (bool) $enable;
}

// Checks wether the template is HTML or not by previously set flag
// Default: true
function isTemplateHtml () {
	// Is the output_mode other than 0 (HTML), then no comments are enabled
	if (getOutputMode() != 0) {
		// No HTML
		return false;
	} else {
		// Maybe HTML?
		return $GLOBALS['is_template_html'];
	}
}

/**
 * Send a HTTP redirect to the browser. This function was taken from DokuWiki
 * (GNU GPL 2; http://www.dokuwiki.org) and modified to fit into mailer project.
 *
 * ----------------------------------------------------------------------------
 * If you want to redirect, please use redirectToUrl(); instead
 * ----------------------------------------------------------------------------
 *
 * Works arround Microsoft IIS cookie sending bug. Does exit the script.
 *
 * @link    http://support.microsoft.com/kb/q176113/
 * @author  Andreas Gohr <andi@splitbrain.org>
 * @access  private
 */
function sendRawRedirect ($url) {
	// always close the session
	session_write_close();

	// Revert entity &amp;
	$url = str_replace('&amp;', '&', $url);

	// check if running on IIS < 6 with CGI-PHP
	if ((isset($_SERVER['SERVER_SOFTWARE'])) && (isset($_SERVER['GATEWAY_INTERFACE'])) &&
		(strpos($_SERVER['GATEWAY_INTERFACE'],'CGI') !== false) &&
		(preg_match('|^Microsoft-IIS/(\d)\.\d$|', trim($_SERVER['SERVER_SOFTWARE']), $matches)) &&
		($matches[1] < 6)) {
		// Send the IIS header
		sendHeader('Refresh: 0;url=' . $url);
	} else {
		// Send generic header
		sendHeader('Location: ' . $url);
	}

	// Shutdown here
	shutdown();
}

// Determines the country of the given user id
function determineCountry ($userid) {
	// Default is 'invalid'
	$country = 'invalid';

	// Is extension country active?
	if (isExtensionActive('country')) {
		// Determine the right country code through the country id
		$id = getUserData('country_code');

		// Then handle it over
		$country = generateCountryInfo($id);
	} else {
		// Get raw code from user data
		$country = getUserData('country');
	}

	// Return it
	return $country;
}

// "Getter" for total confirmed user accounts
function getTotalConfirmedUser () {
	// Is it cached?
	if (!isset($GLOBALS['total_confirmed_users'])) {
		// Then do it
		$GLOBALS['total_confirmed_users'] = countSumTotalData('CONFIRMED', 'user_data', 'userid', 'status', true);
	} // END - if

	// Return cached value
	return $GLOBALS['total_confirmed_users'];
}

// [EOF]
?>
